param(
    [switch]$Verbose,
    [string]$Username = "",
    [string]$Password = "",
    [string]$Domain = "",
    [string]$Server = "LOCALHOST",
    [string]$Port = "35107",
    [string]$ServiceURI = "https://localhost:35113/api",
    [int]$maxDays = 2
)
if($Verbose)
{
    $VerbosePreference = "Continue"
}
# Comment

$MK_CONFDIR = $env:MK_CONFDIR

# Fallback if the (old) agent does not provide the MK_CONFDIR
if (!$MK_CONFDIR) {    
    $MK_CONFDIR= "c:\ProgramData\checkmk\agent\config"
}

# Source the optional configuration file for this agent plugin
$CONFIG_FILE="${MK_CONFDIR}\check_altaro_cfg.ps1"
if (test-path -path "${CONFIG_FILE}" ) {
    . "${CONFIG_FILE}"
}

add-type @"
using System.Net;
using System.Security.Cryptography.X509Certificates;
public class TrustAllCertsPolicy : ICertificatePolicy {
    public bool CheckValidationResult(
        ServicePoint srvPoint, X509Certificate certificate,
        WebRequest request, int certificateProblem) {
        return true;
    }
}
"@
[System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy

function Get-ApiValue($EndPoint, $Type, $Body) {
    
    Write-Verbose "URI: $EndPoint"
    Write-Verbose "Type: $Type"
    Write-Verbose "Body: $Body"

    if ([string]::IsNullOrEmpty($Body))
    {
        $AltaroResult = Invoke-RestMethod -Uri $EndPoint -Method $Type -ContentType "application/json"
    }
    else
    {
        $AltaroResult = Invoke-RestMethod -Uri $EndPoint -Method $Type -ContentType "application/json" -Body (ConvertTo-Json $Body)
    }

    #$AltaroJson = $AltaroResult | ConvertTo-Json
    #$AltaroData = $AltaroJson | ConvertFrom-Json
    
    Write-Verbose "AltaroResult: $AltaroResult"
    #Write-Verbose "AltaroJson: $AltaroJson"
    #Write-Verbose "AltaroData: $AltaroData"

    return $AltaroResult
}

Set-ExecutionPolicy -ExecutionPolicy Unrestricted

if([int]$PSVersionTable.PSVersion.ToString().Substring(0,1) -le 2)
{
    Write-Host "3 Altaro_Backup - PowerShell version " $PSVersionTable.PSVersion "is not supported. Only PowerShell versions 3 and onwards are supported" -Verbose
    Break
}


###############################################################################
###############################################################################
###############################################################################

$AltaroServiceAddress = $ServiceURI
$Nachkommastelle = 2; 

###############################################################################
###############################################################################
###############################################################################

$NewSessionAPIInput = @{
    ServerAddress = $Server; 
    ServerPort = $Port; 
    Username = $Username; 
    Password = $Password; 
    Domain = $Domain;
}

$AltaroNewSessionUri = $AltaroServiceAddress + "/sessions/start"
$AltaroSessionToken = Get-ApiValue $AltaroNewSessionUri Post $NewSessionAPIInput
$sessionToken = $AltaroSessionToken.Data

If ($sessionToken -eq $null)
{
    if($AltaroSessionToken.ErrorCode = "ALTERR_REST_134")
    {
        Write-Host "3 Altaro_Backup - $($AltaroSessionToken.ErrorCode): $($AltaroSessionToken.ErrorMessage)"
    }
    else
    {
        Write-Host "2 Altaro_Backup - $($AltaroSessionToken.ErrorCode): $($AltaroSessionToken.ErrorMessage)"
    }
    Exit 1
}

###############################################################
################   $AltaroSessionTokenUID   ###################
###############################################################
Write-Host "<<<local>>>"

$AltaroApiUri = $AltaroServiceAddress + "/vms/list/" + $sessionToken
$AltaroVMs = Get-ApiValue $AltaroApiUri Get

foreach($vm in $AltaroVMs.VirtualMachines)
{
    $vmName = $vm.VirtualMachineName
    $vmNameSafe = $vmName.Replace(" ","_")
    $vmID = $vm.AltaroVirtualMachineRef
    
    if(!$vm.Configured)
    {
        Write-Host "2 Altaro_$($vmNameSafe) Duration=-1|Size=-1|Time=-1 Nicht eingerichtet"
        continue
    }

    Write-Verbose ""
    Write-Verbose "###############################################################"
    Write-Verbose "###############################################################"
    Write-Verbose "###########  NAME: $($vmName)"
    Write-Verbose "###########  ID: $($vmID)"
    Write-Verbose "###############################################################"
    Write-Verbose "###############################################################"
    
    $date1=[Datetime]::ParseExact($vm.LastBackupTime, 'yyyy-MM-dd-HH-mm-ss', $null)
    $date2 = Get-Date

    if($vm.LastBackupResult -eq "Success" -and (New-TimeSpan -Start $date1 -End $date2).TotalDays -lt $maxDays)
    {
	    $size = [math]::round($vm.LastBackupTransferSizeCompressed/1GB, $Nachkommastelle)
        Write-Host "0 Altaro_$($vmNameSafe) Duration=$($vm.LastBackupDuration)|Size=$($size)|Time=$($vm.LastBackupTime) Next: $($vm.NextBackupTime)"
        
        if($vm.LastOffsiteCopyResult -eq "Success")
        {
            $size = [math]::round($vm.LastOffsiteCopyTransferSizeCompressed/1GB, $Nachkommastelle)
            Write-Host "0 Altaro_$($vmNameSafe)_Offsite Duration=$($vm.LastOffsiteCopyDuration)|Size=$($size)|Time=$($vm.LastOffsiteCopyTime) Next: $($vm.NextOffsiteCopyTime)"
        }
        else
        {
            $AltaroApiUri = $AltaroServiceAddress + "/reports/offsitecopy/" + $sessionToken + "/" + $vmID
            $AltaroData = Get-ApiValue $AltaroApiUri Get
            $size = [math]::round($AltaroData.BackupReports.LastTransferSizeCompressed/1GB, $Nachkommastelle)
            Write-Host "2 Altaro_$($vmNameSafe)_Offsite Duration=$($AltaroData.BackupReports.Duration)|Size=$($size)|Time=$($AltaroData.BackupReports.DateTime) Backup: $($AltaroData.BackupReports.Error)"
        }
    }
    else
    {
        $AltaroApiUri = $AltaroServiceAddress + "/reports/backup/" + $sessionToken + "/" + $vmID
        $AltaroData = Get-ApiValue $AltaroApiUri Get
        
	    $size = [math]::round($AltaroData.BackupReports.LastTransferSizeCompressed/1GB, $Nachkommastelle)
        Write-Host "2 Altaro_$($vmNameSafe) Duration=$($AltaroData.BackupReports.Duration)|Size=$($size)|Time=$($AltaroData.BackupReports.DateTime) Backup: $($AltaroData.BackupReports.Error)"
        
        $AltaroApiUri = $AltaroServiceAddress + "/reports/offsitecopy/" + $sessionToken + "/" + $vmID
        $AltaroData = Get-ApiValue $AltaroApiUri Get
        #$AltaroData
        Write-Host "2 Altaro_$($vmNameSafe)_Offsite - Error in MainBackup"
    }


    #$vm  | ConvertTo-Json | ConvertFrom-Json
}

#$AltaroData.VirtualMachines | ConvertTo-Json | ConvertFrom-Json

###############################################################################
###############################################################################
###############################################################################

$AltaroEndSessionUri = $AltaroServiceAddress + "/sessions/end/" + $sessionToken
$AltaroData = Get-ApiValue $AltaroEndSessionUri Post
$AltaroEndSessionStatusData = $AltaroData.Success

# The Below Statement checks to make sure that session has been sucessfully closed, and if not, notifies the user on next steps. 

If ($AltaroEndSessionStatusData -ne "true")
{
    Write-Host "2 Altaro_Backup - Failed to close the Altaro VM Backup API connection. Please manually run the EndAllSession.ps1 script located in C:\Program Files\Altaro\Altaro Backup\Cmdlets\ on the server running the Altaro VM Backup Software." -Verbose
}

Exit 0

